"use strict";
/*
 See: http://www.adobe.com/devnet-apps/photoshop/fileformatashtml/#50577409_38034
 IPTC application marker is "Photoshop 3.0"
 Signature is '8BIM' Type is 0x0404

 IMAGE RESOURCE BLOCKS:
 Length    Description
 4         Signature: '8BIM'
 2         Unique identifier for the resource. Image resource IDs contains a list of resource IDs used by Photoshop.
 Variable  Name: Pascal string, padded to make the size even (a null name consists of two bytes of 0)
 4         Actual size of resource data that follows
 Variable  The resource data, described in the sections on the individual resource types. It is padded to make the size even.

 */
Object.defineProperty(exports, "__esModule", { value: true });
var IptcFieldMap_1 = require("./IptcFieldMap");
var field_delimiter = 28;
var text_start_marker = 2;
var IptcParser = /** @class */ (function () {
    function IptcParser() {
    }
    /**
     *
     * @param buffer
     * @returns {IptcData}
     * @throws string
     */
    IptcParser.parse = function (buffer) {
        // check for jpeg magic bytes header
        if (buffer[0] != 0xFF || buffer[1] != 0xD8) {
            throw "not a jpeg"; // it is not a valid jpeg
        }
        var offset = 2;
        // Loop through the file looking for the photoshop header bytes
        while (offset < buffer.length) {
            if (buffer[offset] != 0xFF) {
                throw "Not a valid marker at offset " + offset + ", found: " + buffer[offset];
            }
            var applicationMarker = buffer[offset + 1];
            if (applicationMarker == 237) {
                // This is our marker. The content length is 2 byte number.
                return IptcParser.readIPTCData(buffer, offset + 4, buffer.readUInt16BE(offset + 2));
            }
            else {
                // Add header length (2 bytes after header type) to offset
                offset += 2 + buffer.readUInt16BE(offset + 2);
            }
        }
    };
    IptcParser.readIPTCData = function (buffer, start, length) {
        if (start === void 0) { start = 0; }
        if (length === void 0) { length = buffer.length; }
        var data = {};
        if (buffer.slice(start, start + 13).toString("utf-8") != "Photoshop 3.0") {
            throw "Not valid Photoshop data: " + buffer.slice(start, start + 13).toString("utf-8");
        }
        // There are tons of other potentially useful blocks that could be processed here
        // but are currently discarded.
        this.extractBlocks(buffer, start + 13, length).forEach(function (block) {
            // Process IPTC-NAA block 0x0404 (1028)
            if (block.resourceId == 1028) {
                //console.log(block)
                var fields = IptcParser.extractIPTCFieldsFromBlock(buffer, block.startOfBlock, block.sizeOfBlock);
                var date_1, time_1;
                fields.forEach(function (field) {
                    //console.log(field)
                    if (field.id in IptcFieldMap_1.fieldMap) {
                        if (field.id == 55)
                            date_1 = field.value;
                        if (field.id == 60)
                            time_1 = field.value;
                        var name_1 = IptcFieldMap_1.fieldMap[field.id].name.toLowerCase();
                        var val = field.value;
                        if (IptcFieldMap_1.fieldMap[field.id].repeatable) {
                            if (name_1 in data) {
                                data[name_1].push(val);
                            }
                            else {
                                data[name_1] = [val];
                            }
                        }
                        else {
                            data[name_1] = val;
                        }
                    }
                });
                // Construct a real datetime
                if (date_1 && time_1) {
                    try {
                        data['date_time'] = new Date(Date.UTC(parseInt(date_1.slice(0, 4)), parseInt(date_1.slice(4, 6)) - 1, parseInt(date_1.slice(6, 8)), parseInt(time_1.slice(0, 2)), parseInt(time_1.slice(2, 4)), parseInt(time_1.slice(4, 6)), 0));
                    }
                    catch (dateErr) {
                        console.log(dateErr);
                    }
                }
            }
        });
        return data;
    };
    IptcParser.extractIPTCFieldsFromBlock = function (buffer, start, length) {
        var end = Math.min(buffer.length, start + length);
        var data = [];
        for (var i = start; i < end; i++) {
            if (buffer[i] == text_start_marker) {
                // Get the length by finding the next field seperator
                var length_1 = 0;
                while (i + length_1 < end &&
                    buffer[i + length_1] != field_delimiter &&
                    (length_1 < 4 || buffer[i + length_1 + 1] != text_start_marker)) {
                    length_1++;
                }
                //console.log(buffer[i + 1] + ' - ' + length)
                if (length_1 == 0)
                    continue;
                // Convert bytes to string and yield
                data.push({
                    id: buffer[i + 1],
                    value: buffer.slice(i + 4, i + 4 + length_1 - 4).toString("utf-8")
                });
                i += length_1 - 1;
            }
        }
        return data;
    };
    IptcParser.extractBlocks = function (buffer, start, length) {
        var blocks = [];
        var end = Math.min(buffer.length, start + length);
        for (var i = start; i < end; i++) {
            // Signature: '8BIM'
            if (buffer[i] == 56
                && buffer[i + 1] == 66
                && buffer[i + 2] == 73
                && buffer[i + 3] == 77) {
                // Resource ID is 2 bytes, so use u16BE
                var resourceId = buffer.readInt16BE(i + 4);
                // Name: Pascal string, padded to make the size even
                var nameLength = 2;
                // Search for the end of a null-terminated string
                while (nameLength < end - i && buffer[i + 6 + nameLength] != 0x0) {
                    nameLength++;
                }
                var name_2 = buffer.slice(i + 6, i + 6 + nameLength).toString("utf-8");
                var blockSize = buffer.readInt32BE(i + 6 + nameLength);
                blocks.push({
                    resourceId: resourceId,
                    name: name_2,
                    startOfBlock: i + 6 + nameLength + 2,
                    sizeOfBlock: blockSize + 2
                });
            }
        }
        return blocks;
    };
    return IptcParser;
}());
exports.IptcParser = IptcParser;
//# sourceMappingURL=IptcParser.js.map